@@ -428,9 +428,12 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                   port['fixed_ips'])
 
     def update_db_subnet(self, context, id, s, old_pools):
+        subnet = obj_subnet.Subnet.get_object(context, id=id)
+        old_segment_id = subnet.segment_id if subnet else None
         if 'segment_id' in s:
-            self._validate_segment(context, s['network_id'], s['segment_id'],
-                                   action='update')
+            self._validate_segment(
+                context, s['network_id'], s['segment_id'], action='update',
+                old_segment_id=old_segment_id)
         # 'allocation_pools' is removed from 's' in
         # _update_subnet_allocation_pools (ipam_backend_mixin),
         # so create unchanged copy for ipam driver
