@@ -16,9 +16,23 @@
 import functools
 
 import netaddr
+from neutron_lib.api.definitions import l3 as l3_apidef
+from neutron_lib.api.definitions import l3_ext_ha_mode as l3_ext_ha_apidef
+from neutron_lib.api.definitions import port as port_def
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import provider_net as providernet
+from neutron_lib.api import extensions
 from neutron_lib.api import validators
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import priority_group
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
+from neutron_lib.exceptions import l3 as l3_exc
+from neutron_lib.exceptions import l3_ext_ha_mode as l3ha_exc
+from neutron_lib.objects import exceptions as obj_base
+from neutron_lib.plugins import utils as p_utils
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import helpers as log_helpers
@@ -29,65 +43,31 @@ import sqlalchemy as sa
 from sqlalchemy import exc as sql_exc
 from sqlalchemy import orm
 
-from neutron._i18n import _, _LE, _LI, _LW
-from neutron.api.v2 import attributes
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.common import _deprecate
+from neutron._i18n import _
 from neutron.common import constants as n_const
 from neutron.common import utils as n_utils
+from neutron.conf.db import l3_hamode_db
 from neutron.db import _utils as db_utils
 from neutron.db import api as db_api
 from neutron.db.availability_zone import router as router_az_db
 from neutron.db import l3_dvr_db
 from neutron.db.l3_dvr_db import is_distributed_router
-from neutron.db.models import agent as agent_model
-from neutron.db.models import l3 as l3_models
-from neutron.db.models import l3_attrs
 from neutron.db.models import l3ha as l3ha_model
-from neutron.extensions import l3
-from neutron.extensions import l3_ext_ha_mode as l3_ha
-from neutron.extensions import portbindings
-from neutron.extensions import providernet
-from neutron.plugins.common import utils as p_utils
+from neutron.objects import base
+from neutron.objects import l3_hamode
+from neutron.objects import router as l3_obj
 
 
-_deprecate._moved_global('L3HARouterAgentPortBinding', new_module=l3ha_model)
-_deprecate._moved_global('L3HARouterNetwork', new_module=l3ha_model)
-_deprecate._moved_global('L3HARouterVRIdAllocation', new_module=l3ha_model)
-
 VR_ID_RANGE = set(range(1, 255))
 MAX_ALLOCATION_TRIES = 10
 UNLIMITED_AGENTS_PER_ROUTER = 0
 
 LOG = logging.getLogger(__name__)
 
-L3_HA_OPTS = [
-    cfg.BoolOpt('l3_ha',
-                default=False,
-                help=_('Enable HA mode for virtual routers.')),
-    cfg.IntOpt('max_l3_agents_per_router',
-               default=3,
-               help=_("Maximum number of L3 agents which a HA router will be "
-                      "scheduled on. If it is set to 0 then the router will "
-                      "be scheduled on every agent.")),
-    cfg.StrOpt('l3_ha_net_cidr',
-               default=n_const.L3_HA_NET_CIDR,
-               help=_('Subnet used for the l3 HA admin network.')),
-    cfg.StrOpt('l3_ha_network_type', default='',
-               help=_("The network type to use when creating the HA network "
-                      "for an HA router. By default or if empty, the first "
-                      "'tenant_network_types' is used. This is helpful when "
-                      "the VRRP traffic should use a specific network which "
-                      "is not the default one.")),
-    cfg.StrOpt('l3_ha_network_physical_name', default='',
-               help=_("The physical network name with which the HA network "
-                      "can be created."))
-]
-cfg.CONF.register_opts(L3_HA_OPTS)
+l3_hamode_db.register_db_l3_hamode_opts()
 
 
+@registry.has_registry_receivers
 class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                          router_az_db.RouterAvailabilityZoneMixin):
     """Mixin class to add high availability capability to routers."""
@@ -97,9 +77,9 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         try:
             net = netaddr.IPNetwork(self.ha_cidr)
         except netaddr.AddrFormatError:
-            raise l3_ha.HANetworkCIDRNotValid(cidr=self.ha_cidr)
+            raise l3ha_exc.HANetworkCIDRNotValid(cidr=self.ha_cidr)
         if ('/' not in self.ha_cidr or net.network != net.ip):
-            raise l3_ha.HANetworkCIDRNotValid(cidr=self.ha_cidr)
+            raise l3ha_exc.HANetworkCIDRNotValid(cidr=self.ha_cidr)
 
         self._check_num_agents_per_router()
 
@@ -107,41 +87,24 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         max_agents = cfg.CONF.max_l3_agents_per_router
 
         if max_agents != UNLIMITED_AGENTS_PER_ROUTER and max_agents < 1:
-            raise l3_ha.HAMaximumAgentsNumberNotValid(max_agents=max_agents)
+            raise l3ha_exc.HAMaximumAgentsNumberNotValid(max_agents=max_agents)
 
     def __new__(cls, *args, **kwargs):
         inst = super(L3_HA_NAT_db_mixin, cls).__new__(cls, *args, **kwargs)
         inst._verify_configuration()
-        registry.subscribe(inst._release_router_vr_id,
-                           resources.ROUTER, events.PRECOMMIT_DELETE)
-        registry.subscribe(inst._cleanup_ha_network,
-                           resources.ROUTER, events.AFTER_DELETE)
-        registry.subscribe(inst._precommit_router_create,
-                           resources.ROUTER, events.PRECOMMIT_CREATE)
-        registry.subscribe(inst._before_router_create,
-                           resources.ROUTER, events.BEFORE_CREATE)
-        registry.subscribe(inst._after_router_create,
-                           resources.ROUTER, events.AFTER_CREATE)
-        registry.subscribe(inst._validate_migration,
-                           resources.ROUTER, events.PRECOMMIT_UPDATE)
-        registry.subscribe(inst._reconfigure_ha_resources,
-                           resources.ROUTER, events.AFTER_UPDATE)
         return inst
 
     def get_ha_network(self, context, tenant_id):
-        return (context.session.query(l3ha_model.L3HARouterNetwork).
-                filter(l3ha_model.L3HARouterNetwork.tenant_id == tenant_id).
-                first())
+        pager = base.Pager(limit=1)
+        results = l3_hamode.L3HARouterNetwork.get_objects(
+            context, _pager=pager, project_id=tenant_id)
+        return results.pop() if results else None
 
     def _get_allocated_vr_id(self, context, network_id):
-        with context.session.begin(subtransactions=True):
-            query = (context.session.query(
-                l3ha_model.L3HARouterVRIdAllocation).
-                filter(l3ha_model.L3HARouterVRIdAllocation.network_id ==
-                       network_id))
-
-            allocated_vr_ids = set(a.vr_id for a in query) - set([0])
+        vr_id_objs = l3_hamode.L3HARouterVRIdAllocation.get_objects(
+            context, network_id=network_id)
 
+        allocated_vr_ids = set(a.vr_id for a in vr_id_objs) - set([0])
         return allocated_vr_ids
 
     @db_api.retry_if_session_inactive()
@@ -164,42 +127,50 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                              'ha_vr_id': router_db.extra_attributes.ha_vr_id})
                         return
 
+                    old_router = self._make_router_dict(router_db)
                     allocated_vr_ids = self._get_allocated_vr_id(context,
                                                                  network_id)
                     available_vr_ids = VR_ID_RANGE - allocated_vr_ids
 
                     if not available_vr_ids:
-                        raise l3_ha.NoVRIDAvailable(router_id=router_id)
+                        raise l3ha_exc.NoVRIDAvailable(router_id=router_id)
 
-                    allocation = l3ha_model.L3HARouterVRIdAllocation()
-                    allocation.network_id = network_id
-                    allocation.vr_id = available_vr_ids.pop()
+                    allocation = l3_hamode.L3HARouterVRIdAllocation(
+                        context, network_id=network_id,
+                        vr_id=available_vr_ids.pop())
+                    allocation.create()
 
-                    context.session.add(allocation)
                     router_db.extra_attributes.ha_vr_id = allocation.vr_id
                     LOG.debug(
                         "Router %(router_id)s has been allocated a ha_vr_id "
                         "%(ha_vr_id)d.",
                         {'router_id': router_id, 'ha_vr_id': allocation.vr_id})
+                    router_body = {l3_apidef.ROUTER:
+                            {l3_ext_ha_apidef.HA_INFO: True,
+                             'ha_vr_id': allocation.vr_id}}
+                    registry.publish(resources.ROUTER, events.PRECOMMIT_UPDATE,
+                                     self, payload=events.DBEventPayload(
+                                         context, request_body=router_body,
+                                         states=(old_router,),
+                                         resource_id=router_id,
+                                         desired_state=router_db))
 
                     return allocation.vr_id
 
-            except db_exc.DBDuplicateEntry:
-                LOG.info(_LI("Attempt %(count)s to allocate a VRID in the "
-                             "network %(network)s for the router %(router)s"),
+            except obj_base.NeutronDbObjectDuplicateEntry:
+                LOG.info("Attempt %(count)s to allocate a VRID in the "
+                         "network %(network)s for the router %(router)s",
                          {'count': count, 'network': network_id,
                           'router': router_id})
 
-        raise l3_ha.MaxVRIDAllocationTriesReached(
+        raise l3ha_exc.MaxVRIDAllocationTriesReached(
             network_id=network_id, router_id=router_id,
             max_tries=MAX_ALLOCATION_TRIES)
 
     @db_api.retry_if_session_inactive()
     def _delete_vr_id_allocation(self, context, ha_network, vr_id):
-        with context.session.begin(subtransactions=True):
-            context.session.query(
-                l3ha_model.L3HARouterVRIdAllocation).filter_by(
-                    network_id=ha_network.network_id, vr_id=vr_id).delete()
+        l3_hamode.L3HARouterVRIdAllocation.delete_objects(
+            context, network_id=ha_network.network_id, vr_id=vr_id)
 
     def _create_ha_subnet(self, context, network_id, tenant_id):
         args = {'network_id': network_id,
@@ -214,21 +185,18 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
 
     def _create_ha_network_tenant_binding(self, context, tenant_id,
                                           network_id):
-        with context.session.begin():
-            ha_network = l3ha_model.L3HARouterNetwork(
-                tenant_id=tenant_id, network_id=network_id)
-            context.session.add(ha_network)
+        ha_network = l3_hamode.L3HARouterNetwork(
+            context, project_id=tenant_id, network_id=network_id)
+        ha_network.create()
         # we need to check if someone else just inserted at exactly the
         # same time as us because there is no constrain in L3HARouterNetwork
         # that prevents multiple networks per tenant
-        with context.session.begin(subtransactions=True):
-            items = (context.session.query(l3ha_model.L3HARouterNetwork).
-                     filter_by(tenant_id=tenant_id).all())
-            if len(items) > 1:
-                # we need to throw an error so our network is deleted
-                # and the process is started over where the existing
-                # network will be selected.
-                raise db_exc.DBDuplicateEntry(columns=['tenant_id'])
+        if l3_hamode.L3HARouterNetwork.count(
+                context, project_id=tenant_id) > 1:
+            # we need to throw an error so our network is deleted
+            # and the process is started over where the existing
+            # network will be selected.
+            raise db_exc.DBDuplicateEntry(columns=['tenant_id'])
         return ha_network
 
     def _add_ha_network_settings(self, network):
@@ -274,9 +242,9 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         max_agents = cfg.CONF.max_l3_agents_per_router
         if max_agents:
             if max_agents > num_agents:
-                LOG.info(_LI("Number of active agents lower than "
-                             "max_l3_agents_per_router. L3 agents "
-                             "available: %s"), num_agents)
+                LOG.info("Number of active agents lower than "
+                         "max_l3_agents_per_router. L3 agents "
+                         "available: %s", num_agents)
             else:
                 num_agents = max_agents
 
@@ -286,10 +254,11 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def _create_ha_port_binding(self, context, router_id, port_id):
         try:
             with context.session.begin():
-                routerportbinding = l3_models.RouterPort(
-                    port_id=port_id, router_id=router_id,
-                    port_type=constants.DEVICE_OWNER_ROUTER_HA_INTF)
-                context.session.add(routerportbinding)
+                l3_obj.RouterPort(
+                    context,
+                    port_id=port_id,
+                    router_id=router_id,
+                    port_type=constants.DEVICE_OWNER_ROUTER_HA_INTF).create()
                 portbinding = l3ha_model.L3HARouterAgentPortBinding(
                     port_id=port_id, router_id=router_id)
                 context.session.add(portbinding)
@@ -303,7 +272,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                         'Failed to create HA router agent PortBinding, '
                         'Router %s has already been removed '
                         'by concurrent operation', router_id)
-                    raise l3.RouterNotFound(router_id=router_id)
+                    raise l3_exc.RouterNotFound(router_id=router_id)
 
     def add_ha_port(self, context, router_id, network_id, tenant_id):
         # NOTE(kevinbenton): we have to block any ongoing transactions because
@@ -387,6 +356,8 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         return n_utils.create_object_with_dependency(
             creator, dep_getter, dep_creator, dep_id_attr, dep_deleter)[1]
 
+    @registry.receives(resources.ROUTER, [events.BEFORE_CREATE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     @db_api.retry_if_session_inactive()
     def _before_router_create(self, resource, event, trigger,
                               context, router, **kwargs):
@@ -399,6 +370,8 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         if not self.get_ha_network(context, router['tenant_id']):
             self._create_ha_network(context, router['tenant_id'])
 
+    @registry.receives(resources.ROUTER, [events.PRECOMMIT_CREATE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _precommit_router_create(self, resource, event, trigger, context,
                                  router, router_db, **kwargs):
         """Event handler to set ha flag and status on creation."""
@@ -414,9 +387,11 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         if not ha_net:
             # net was deleted, throw a retry to start over to create another
             raise db_exc.RetryRequest(
-                    l3_ha.HANetworkConcurrentDeletion(
+                l3ha_exc.HANetworkConcurrentDeletion(
                         tenant_id=router['tenant_id']))
 
+    @registry.receives(resources.ROUTER, [events.AFTER_CREATE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _after_router_create(self, resource, event, trigger, context,
                              router_id, router, router_db, **kwargs):
         if not router['ha']:
@@ -427,30 +402,30 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             self._notify_router_updated(context, router_id)
         except Exception as e:
             with excutils.save_and_reraise_exception() as ctx:
-                if isinstance(e, l3_ha.NoVRIDAvailable):
+                if isinstance(e, l3ha_exc.NoVRIDAvailable):
                     ctx.reraise = False
-                    LOG.warning(_LW("No more VRIDs for router: %s"), e)
+                    LOG.warning("No more VRIDs for router: %s", e)
                 else:
-                    LOG.exception(_LE("Failed to schedule HA router %s."),
+                    LOG.exception("Failed to schedule HA router %s.",
                                   router_id)
                 router['status'] = self._update_router_db(
                     context, router_id,
-                    {'status': n_const.ROUTER_STATUS_ERROR})['status']
+                    {'status': constants.ERROR})['status']
 
-    def _validate_migration(self, resource, event, trigger, context,
-                            router_id, router, router_db, old_router,
-                            **kwargs):
+    @registry.receives(resources.ROUTER, [events.PRECOMMIT_UPDATE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
+    def _validate_migration(self, resource, event, trigger, payload=None):
         """Event handler on precommit update to validate migration."""
 
-        original_ha_state = old_router['ha']
-        requested_ha_state = router.get('ha')
+        original_ha_state = payload.states[0]['ha']
+        requested_ha_state = payload.request_body.get('ha')
 
         ha_changed = (requested_ha_state is not None and
                       requested_ha_state != original_ha_state)
         if not ha_changed:
             return
 
-        if router_db.admin_state_up:
+        if payload.desired_state.admin_state_up:
             msg = _('Cannot change HA attribute of active routers. Please '
                     'set router admin_state_up to False prior to upgrade')
             raise n_exc.BadRequest(resource='router', msg=msg)
@@ -458,16 +433,35 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         if requested_ha_state:
             # This will throw HANotEnoughAvailableAgents if there aren't
             # enough l3 agents to handle this router.
-            self.get_number_of_agents_for_scheduling(context)
+            self.get_number_of_agents_for_scheduling(payload.context)
         else:
 
-            ha_network = self.get_ha_network(context,
-                                             router_db.tenant_id)
+            ha_network = self.get_ha_network(payload.context,
+                                             payload.desired_state.tenant_id)
             self._delete_vr_id_allocation(
-                context, ha_network, router_db.extra_attributes.ha_vr_id)
-            router_db.extra_attributes.ha_vr_id = None
-        self.set_extra_attr_value(context, router_db, 'ha', requested_ha_state)
+                payload.context, ha_network,
+                payload.desired_state.extra_attributes.ha_vr_id)
+            payload.desired_state.extra_attributes.ha_vr_id = None
+        if (payload.request_body.get('distributed') or
+                payload.states[0]['distributed']):
+            self.set_extra_attr_value(payload.context, payload.desired_state,
+                                      'ha', requested_ha_state)
+            return
+        if requested_ha_state:
+            self._migrate_router_ports(
+                payload.context, payload.desired_state,
+                old_owner=constants.DEVICE_OWNER_ROUTER_INTF,
+                new_owner=constants.DEVICE_OWNER_HA_REPLICATED_INT)
+        else:
+            self._migrate_router_ports(
+                payload.context, payload.desired_state,
+                old_owner=constants.DEVICE_OWNER_HA_REPLICATED_INT,
+                new_owner=constants.DEVICE_OWNER_ROUTER_INTF)
+        self.set_extra_attr_value(
+            payload.context, payload.desired_state, 'ha', requested_ha_state)
 
+    @registry.receives(resources.ROUTER, [events.AFTER_UPDATE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _reconfigure_ha_resources(self, resource, event, trigger, context,
                                   router_id, old_router, router, router_db,
                                   **kwargs):
@@ -491,10 +485,6 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             if ha_network:
                 self.safe_delete_ha_network(context, ha_network,
                                             router_db.tenant_id)
-            self._migrate_router_ports(
-                context, router_db,
-                old_owner=constants.DEVICE_OWNER_HA_REPLICATED_INT,
-                new_owner=constants.DEVICE_OWNER_ROUTER_INTF)
 
         self.schedule_router(context, router_id)
         self._notify_router_updated(context, router_db.id)
@@ -515,17 +505,19 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             LOG.debug(
                 "HA network for tenant %s was already deleted.", tenant_id)
         except sa.exc.InvalidRequestError:
-            LOG.info(_LI("HA network %s can not be deleted."), net_id)
+            LOG.info("HA network %s can not be deleted.", net_id)
         except n_exc.NetworkInUse:
             # network is still in use, this is normal so we don't
             # log anything
             pass
         else:
-            LOG.info(_LI("HA network %(network)s was deleted as "
-                         "no HA routers are present in tenant "
-                         "%(tenant)s."),
+            LOG.info("HA network %(network)s was deleted as "
+                     "no HA routers are present in tenant "
+                     "%(tenant)s.",
                      {'network': net_id, 'tenant': tenant_id})
 
+    @registry.receives(resources.ROUTER, [events.PRECOMMIT_DELETE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _release_router_vr_id(self, resource, event, trigger, context,
                               router_db, **kwargs):
         """Event handler for removal of VRID during router delete."""
@@ -536,6 +528,8 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 self._delete_vr_id_allocation(
                     context, ha_network, router_db.extra_attributes.ha_vr_id)
 
+    @registry.receives(resources.ROUTER, [events.AFTER_DELETE],
+                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     @db_api.retry_if_session_inactive()
     def _cleanup_ha_network(self, resource, event, trigger, context,
                             router_id, original, **kwargs):
@@ -556,24 +550,14 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def get_ha_router_port_bindings(self, context, router_ids, host=None):
         if not router_ids:
             return []
-        query = context.session.query(l3ha_model.L3HARouterAgentPortBinding)
-
-        if host:
-            query = query.join(agent_model.Agent).filter(
-                agent_model.Agent.host == host)
-
-        query = query.filter(
-            l3ha_model.L3HARouterAgentPortBinding.router_id.in_(router_ids))
-
-        return query.all()
+        return (
+            l3_hamode.L3HARouterAgentPortBinding.get_l3ha_filter_host_router(
+                context, router_ids, host))
 
     @staticmethod
     def _check_router_agent_ha_binding(context, router_id, agent_id):
-        query = context.session.query(l3ha_model.L3HARouterAgentPortBinding)
-        query = query.filter(
-            l3ha_model.L3HARouterAgentPortBinding.router_id == router_id,
-            l3ha_model.L3HARouterAgentPortBinding.l3_agent_id == agent_id)
-        return query.first() is not None
+        return l3_hamode.L3HARouterAgentPortBinding.objects_exist(
+            context, router_id=router_id, l3_agent_id=agent_id)
 
     def _get_bindings_and_update_router_state_for_dead_agents(self, context,
                                                               router_id):
@@ -583,20 +567,36 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         """
         with context.session.begin(subtransactions=True):
             bindings = self.get_ha_router_port_bindings(context, [router_id])
-            dead_agents = []
-            active = [binding for binding in bindings
-                      if binding.state == n_const.HA_ROUTER_STATE_ACTIVE]
-            # Check dead agents only if we have more then one active agent
-            if len(active) > 1:
-                dead_agents = [binding.agent for binding in active
-                               if not (binding.agent.is_active and
-                                       binding.agent.admin_state_up)]
-                for dead_agent in dead_agents:
-                    self.update_routers_states(
-                        context,
-                        {router_id: n_const.HA_ROUTER_STATE_STANDBY},
-                        dead_agent.host)
-        if dead_agents:
+            router_active_agents_dead = []
+            router_standby_agents_dead = []
+            # List agents where router is active and agent is dead
+            # and agents where router is standby and agent is dead
+            for binding in bindings:
+                if not (binding.agent.is_active and
+                        binding.agent.admin_state_up):
+                    if binding.state == n_const.HA_ROUTER_STATE_ACTIVE:
+                        router_active_agents_dead.append(binding.agent)
+                    elif binding.state == n_const.HA_ROUTER_STATE_STANDBY:
+                        router_standby_agents_dead.append(binding.agent)
+            if router_active_agents_dead:
+                # Just check if all l3_agents are down
+                # then assuming some communication issue
+                if (len(router_active_agents_dead) +
+                        len(router_standby_agents_dead) == len(bindings)):
+                    # Make router status as unknown because
+                    # agent communication may be issue but router
+                    # may still be active. We do not know the
+                    # exact status of router.
+                    state = n_const.HA_ROUTER_STATE_UNKNOWN
+                else:
+                    # Make router status as standby on all dead agents
+                    # as some other agents are alive , router can become
+                    # active on them after some time
+                    state = n_const.HA_ROUTER_STATE_STANDBY
+                for dead_agent in router_active_agents_dead:
+                    self.update_routers_states(context, {router_id: state},
+                                               dead_agent.host)
+        if router_active_agents_dead:
             return self.get_ha_router_port_bindings(context, [router_id])
         return bindings
 
@@ -623,7 +623,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             None)
 
     @log_helpers.log_method_call
-    def _process_sync_ha_data(self, context, routers, host, agent_mode):
+    def _process_sync_ha_data(self, context, routers, host, is_any_dvr_agent):
         routers_dict = dict((router['id'], router) for router in routers)
 
         bindings = self.get_ha_router_port_bindings(context,
@@ -633,8 +633,8 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             port = binding.port
             if not port:
                 # Filter the HA router has no ha port here
-                LOG.info(_LI("HA router %s is missing HA router port "
-                             "bindings. Skipping it."),
+                LOG.info("HA router %s is missing HA router port "
+                         "bindings. Skipping it.",
                          binding.router_id)
                 routers_dict.pop(binding.router_id)
                 continue
@@ -652,20 +652,22 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
 
         self._populate_mtu_and_subnets_for_ports(context, interfaces)
 
-        # If this is a DVR+HA router, but the agent is question is in 'dvr'
-        # mode (as opposed to 'dvr_snat'), then we want to always return it
-        # even though it's missing the '_ha_interface' key.
+        # If this is a DVR+HA router, then we want to always return it even
+        # though it's missing the '_ha_interface' key. The agent will have
+        # to figure out what kind of router setup is needed.
         return [r for r in list(routers_dict.values())
-                if (agent_mode == constants.L3_AGENT_MODE_DVR or
+                if (is_any_dvr_agent or
                     not r.get('ha') or r.get(constants.HA_INTERFACE_KEY))]
 
     @log_helpers.log_method_call
     def get_ha_sync_data_for_host(self, context, host, agent,
                                   router_ids=None, active=None):
         agent_mode = self._get_agent_mode(agent)
-        dvr_agent_mode = (agent_mode in [constants.L3_AGENT_MODE_DVR_SNAT,
-                                         constants.L3_AGENT_MODE_DVR])
-        if (dvr_agent_mode and n_utils.is_extension_supported(
+        dvr_agent_mode = (
+            agent_mode in [constants.L3_AGENT_MODE_DVR_SNAT,
+                           constants.L3_AGENT_MODE_DVR,
+                           constants.L3_AGENT_MODE_DVR_NO_EXTERNAL])
+        if (dvr_agent_mode and extensions.is_extension_supported(
                 self, constants.L3_DISTRIBUTED_EXT_ALIAS)):
             # DVR has to be handled differently
             sync_data = self._get_dvr_sync_data(context, host, agent,
@@ -673,7 +675,8 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         else:
             sync_data = super(L3_HA_NAT_db_mixin, self).get_sync_data(context,
                                                             router_ids, active)
-        return self._process_sync_ha_data(context, sync_data, host, agent_mode)
+        return self._process_sync_ha_data(
+            context, sync_data, host, dvr_agent_mode)
 
     @classmethod
     def _set_router_states(cls, context, bindings, states):
@@ -706,15 +709,50 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             if states[port['device_id']] == n_const.HA_ROUTER_STATE_ACTIVE)
 
         for port in active_ports:
-            port[portbindings.HOST_ID] = host
             try:
-                self._core_plugin.update_port(admin_ctx, port['id'],
-                                              {attributes.PORT: port})
+                self._core_plugin.update_port(
+                    admin_ctx, port['id'],
+                    {port_def.RESOURCE_NAME: {portbindings.HOST_ID: host}})
             except (orm.exc.StaleDataError, orm.exc.ObjectDeletedError,
                     n_exc.PortNotFound):
                 # Take concurrently deleted interfaces in to account
                 pass
 
+    def _get_gateway_port_host(self, context, router, gw_ports):
+        if not router.get('ha'):
+            return super(L3_HA_NAT_db_mixin, self)._get_gateway_port_host(
+                context, router, gw_ports)
+
+        gw_port_id = router['gw_port_id']
+        gateway_port = gw_ports.get(gw_port_id)
+        if not gw_port_id or not gateway_port:
+            return
+        gateway_port_status = gateway_port['status']
+        gateway_port_binding_host = gateway_port[portbindings.HOST_ID]
+
+        admin_ctx = context.elevated()
+        router_id = router['id']
+        ha_bindings = self.get_l3_bindings_hosting_router_with_ha_states(
+            admin_ctx, router_id)
+        LOG.debug("HA router %(router_id)s gateway port %(gw_port_id)s "
+                  "binding host: %(host)s, status: %(status)s",
+                  {"router_id": router_id,
+                   "gw_port_id": gateway_port['id'],
+                   "host": gateway_port_binding_host,
+                   "status": gateway_port_status})
+        for ha_binding_agent, ha_binding_state in ha_bindings:
+            if ha_binding_state != n_const.HA_ROUTER_STATE_ACTIVE:
+                continue
+            # For create router gateway, the gateway port may not be ACTIVE
+            # yet, so we return 'master' host directly.
+            if gateway_port_status != constants.PORT_STATUS_ACTIVE:
+                return ha_binding_agent.host
+            # Do not let the original 'master' (current is backup) host,
+            # override the gateway port binding host.
+            if (gateway_port_status == constants.PORT_STATUS_ACTIVE and
+                    ha_binding_agent.host == gateway_port_binding_host):
+                return ha_binding_agent.host
+
 
 def is_ha_router(router):
     """Return True if router to be handled is ha."""
@@ -730,17 +768,10 @@ def is_ha_router(router):
 
 
 def is_ha_router_port(context, device_owner, router_id):
-    session = db_api.get_reader_session()
     if device_owner == constants.DEVICE_OWNER_HA_REPLICATED_INT:
         return True
     elif device_owner == constants.DEVICE_OWNER_ROUTER_SNAT:
-        query = session.query(l3_attrs.RouterExtraAttributes)
-        query = query.filter_by(ha=True)
-        query = query.filter(l3_attrs.RouterExtraAttributes.router_id ==
-                             router_id)
-        return bool(query.limit(1).count())
+        return l3_obj.RouterExtraAttributes.objects_exist(
+            context, router_id=router_id, ha=True)
     else:
         return False
-
-
-_deprecate._MovedGlobals()
