@@ -506,3 +506,13 @@ class Port(base.NeutronDbObject):
             sg_models.SecurityGroupPortBinding.security_group_id.in_(
                 security_group_ids))
         return [port_binding['port_id'] for port_binding in query.all()]
+
+    @classmethod
+    def get_ports_by_binding_type_and_host(cls, context,
+                                           binding_type, host):
+        query = context.session.query(models_v2.Port).join(
+            ml2_models.PortBinding)
+        query = query.filter(
+            ml2_models.PortBinding.vif_type == binding_type,
+            ml2_models.PortBinding.host == host)
+        return [cls._load_object(context, db_obj) for db_obj in query.all()]
