@@ -37,6 +37,7 @@ def nonzero(f):
     else:
         return f.__nonzero__()
 
+
 # Useful global dummy variables.
 NET_UUID = '3faeebfe-5d37-11e1-a64b-000c29d5f0a7'
 LS_ID = 420
@@ -182,13 +183,10 @@ class TunnelTest(object):
             ovs_lib.BaseOVS,
             'get_bridge_external_bridge_id').start()
         self.get_bridge_external_bridge_id.side_effect = (
-            lambda bridge, log_errors: bridge if bridge in self.ovs_bridges
-            else None)
+            lambda bridge: bridge if bridge in self.ovs_bridges else None)
 
         self.execute = mock.patch('neutron.agent.common.utils.execute').start()
-        self.mock_check_bridge_datapath_id = mock.patch.object(
-            self.mod_agent.OVSNeutronAgent,
-            '_check_bridge_datapath_id').start()
+
         self._define_expected_calls()
 
     def _define_expected_calls(self, arp_responder=False):
@@ -573,12 +571,12 @@ class TunnelTest(object):
             mock.call.cleanup_flows(),
             mock.call.check_canary_table()
         ]
-        self.mock_map_tun_bridge_expected += [
-            mock.call.cleanup_flows(),
-        ]
         self.mock_tun_bridge_expected += [
             mock.call.cleanup_flows()
         ]
+        self.mock_map_tun_bridge_expected += [
+            mock.call.cleanup_flows()
+        ]
         # No cleanup is expected on ancillary bridge
 
         self.ovs_bridges[self.INT_BRIDGE].check_canary_table.return_value = \
