@@ -13,26 +13,27 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.agent import topics
+from neutron_lib.api.definitions import port_security as psec
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
 from neutron_lib.plugins import directory
+from neutron_lib.plugins.ml2 import api
+from neutron_lib.services.qos import constants as qos_consts
 from oslo_log import log
 import oslo_messaging
 from sqlalchemy.orm import exc
 
-from neutron._i18n import _LE, _LW
+from neutron.agent import _topics as n_topics
 from neutron.api.rpc.handlers import dvr_rpc
 from neutron.api.rpc.handlers import securitygroups_rpc as sg_rpc
-from neutron.callbacks import resources
+from neutron.common import constants as c_const
 from neutron.common import rpc as n_rpc
-from neutron.common import topics
 from neutron.db import l3_hamode_db
 from neutron.db import provisioning_blocks
-from neutron.extensions import portbindings
-from neutron.extensions import portsecurity as psec
 from neutron.plugins.ml2 import db as ml2_db
-from neutron.plugins.ml2 import driver_api as api
 from neutron.plugins.ml2.drivers import type_tunnel
-from neutron.services.qos import qos_consts
 # REVISIT(kmestery): Allow the type and mechanism drivers to supply the
 # mixins and eventually remove the direct dependencies on type_tunnel.
 
@@ -57,6 +58,14 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
         self.setup_tunnel_callback_mixin(notifier, type_manager)
         super(RpcCallbacks, self).__init__()
 
+    def _get_new_status(self, host, port_context):
+        port = port_context.current
+        if not host or host == port_context.host:
+            new_status = (n_const.PORT_STATUS_BUILD if port['admin_state_up']
+                          else n_const.PORT_STATUS_DOWN)
+            if port['status'] != new_status:
+                return new_status
+
     def get_device_details(self, rpc_context, **kwargs):
         """Agent requests device details."""
         agent_id = kwargs.get('agent_id')
@@ -87,36 +96,43 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
             if port['network_id'] not in cached_networks:
                 cached_networks[port['network_id']] = (
                     port_context.network.current)
-        return self._get_device_details(rpc_context, agent_id=agent_id,
-                                        host=host, device=device,
-                                        port_context=port_context)
+        result = self._get_device_details(rpc_context, agent_id=agent_id,
+                                          host=host, device=device,
+                                          port_context=port_context)
+        if 'network_id' in result:
+            # success so we update status
+            new_status = self._get_new_status(host, port_context)
+            if new_status:
+                plugin.update_port_status(rpc_context,
+                                          port_id,
+                                          new_status,
+                                          host,
+                                          port_context.network.current)
+        return result
 
     def _get_device_details(self, rpc_context, agent_id, host, device,
                             port_context):
         segment = port_context.bottom_bound_segment
         port = port_context.current
-        plugin = directory.get_plugin()
-        port_id = port_context.current['id']
 
         if not segment:
-            LOG.warning(_LW("Device %(device)s requested by agent "
-                            "%(agent_id)s on network %(network_id)s not "
-                            "bound, vif_type: %(vif_type)s"),
+            LOG.warning("Device %(device)s requested by agent "
+                        "%(agent_id)s on network %(network_id)s not "
+                        "bound, vif_type: %(vif_type)s",
                         {'device': device,
                          'agent_id': agent_id,
                          'network_id': port['network_id'],
                          'vif_type': port_context.vif_type})
             return {'device': device}
 
-        if (not host or host == port_context.host):
-            new_status = (n_const.PORT_STATUS_BUILD if port['admin_state_up']
-                          else n_const.PORT_STATUS_DOWN)
-            if port['status'] != new_status:
-                plugin.update_port_status(rpc_context,
-                                          port_id,
-                                          new_status,
-                                          host,
-                                          port_context.network.current)
+        if (port['device_owner'].startswith(
+                n_const.DEVICE_OWNER_COMPUTE_PREFIX) and
+                port[portbindings.HOST_ID] != host):
+            LOG.debug("Device %(device)s has no active binding in host "
+                      "%(host)s", {'device': device,
+                                   'host': host})
+            return {'device': device,
+                    c_const.NO_ACTIVE_BINDING: True}
 
         network_qos_policy_id = port_context.network._network.get(
             qos_consts.QOS_POLICY_ID)
@@ -180,9 +196,19 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
                                device=device,
                                port_context=bound_contexts[device]))
             except Exception:
-                LOG.exception(_LE("Failed to get details for device %s"),
+                LOG.exception("Failed to get details for device %s",
                               device)
                 failed_devices.append(device)
+        new_status_map = {ctxt.current['id']: self._get_new_status(host, ctxt)
+                          for ctxt in bound_contexts.values() if ctxt}
+        # filter out any without status changes
+        new_status_map = {p: s for p, s in new_status_map.items() if s}
+        try:
+            plugin.update_port_statuses(rpc_context, new_status_map, host)
+        except Exception:
+            LOG.exception("Failure updating statuses, retrying all")
+            failed_devices = devices_to_fetch
+            devices = []
 
         return {'devices': devices,
                 'failed_devices': failed_devices}
@@ -214,8 +240,8 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
                           "executed concurrently. Ignoring StaleDataError.")
                 return {'device': device,
                         'exists': port_exists}
-        self.notify_ha_port_status(port_id, rpc_context,
-                                   n_const.PORT_STATUS_DOWN, host)
+        self.notify_l2pop_port_wiring(port_id, rpc_context,
+                                      n_const.PORT_STATUS_DOWN, host)
 
         return {'device': device,
                 'exists': port_exists}
@@ -250,8 +276,8 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
                     return
         else:
             self.update_port_status_to_active(port, rpc_context, port_id, host)
-        self.notify_ha_port_status(port_id, rpc_context,
-                                   n_const.PORT_STATUS_ACTIVE, host, port=port)
+        self.notify_l2pop_port_wiring(port_id, rpc_context,
+                                      n_const.PORT_STATUS_ACTIVE, host)
 
     def update_port_status_to_active(self, port, rpc_context, port_id, host):
         plugin = directory.get_plugin()
@@ -274,29 +300,50 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
                 rpc_context, port['id'], resources.PORT,
                 provisioning_blocks.L2_AGENT_ENTITY)
 
-    def notify_ha_port_status(self, port_id, rpc_context,
-                              status, host, port=None):
+    def notify_l2pop_port_wiring(self, port_id, rpc_context,
+                                 status, host):
+        """Notify the L2pop driver that a port has been wired/unwired.
+
+        The L2pop driver uses this notification to broadcast forwarding
+        entries to other agents on the same network as the port for port_id.
+        """
         plugin = directory.get_plugin()
         l2pop_driver = plugin.mechanism_manager.mech_drivers.get(
                 'l2population')
         if not l2pop_driver:
             return
+        port = ml2_db.get_port(rpc_context, port_id)
         if not port:
-            port = ml2_db.get_port(rpc_context, port_id)
-            if not port:
+            return
+        port_context = plugin.get_bound_port_context(
+                rpc_context, port_id, host)
+        if not port_context:
+            # port deleted
+            return
+        # NOTE: DVR ports are already handled and updated through l2pop
+        # and so we don't need to update it again here. But, l2pop did not
+        # handle DVR ports while restart neutron-*-agent, we need to handle
+        # it here.
+        if (port['device_owner'] == n_const.DEVICE_OWNER_DVR_INTERFACE and
+                not l2pop_driver.obj.agent_restarted(port_context)):
+            return
+        port = port_context.current
+        if (port['device_owner'] != n_const.DEVICE_OWNER_DVR_INTERFACE and
+            status == n_const.PORT_STATUS_ACTIVE and
+            port[portbindings.HOST_ID] != host and
+            not l3_hamode_db.is_ha_router_port(rpc_context,
+                                               port['device_owner'],
+                                               port['device_id'])):
+                # don't setup ACTIVE forwarding entries unless bound to this
+                # host or if it's an HA or DVR port (which is special-cased in
+                # the mech driver)
                 return
-        is_ha_port = l3_hamode_db.is_ha_router_port(rpc_context,
-                                                    port['device_owner'],
-                                                    port['device_id'])
-        if is_ha_port:
-            port_context = plugin.get_bound_port_context(
-                    rpc_context, port_id)
-            port_context.current['status'] = status
-            port_context.current[portbindings.HOST_ID] = host
-            if status == n_const.PORT_STATUS_ACTIVE:
-                l2pop_driver.obj.update_port_up(port_context)
-            else:
-                l2pop_driver.obj.update_port_down(port_context)
+        port_context.current['status'] = status
+        port_context.current[portbindings.HOST_ID] = host
+        if status == n_const.PORT_STATUS_ACTIVE:
+            l2pop_driver.obj.update_port_up(port_context)
+        else:
+            l2pop_driver.obj.update_port_down(port_context)
 
     def update_device_list(self, rpc_context, **kwargs):
         devices_up = []
@@ -313,7 +360,7 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
                         **kwargs)
                 except Exception:
                     failed_devices_up.append(device)
-                    LOG.error(_LE("Failed to update device %s up"), device)
+                    LOG.error("Failed to update device %s up", device)
                 else:
                     devices_up.append(device)
 
@@ -327,7 +374,7 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
                         **kwargs)
                 except Exception:
                     failed_devices_down.append(device)
-                    LOG.error(_LE("Failed to update device %s down"), device)
+                    LOG.error("Failed to update device %s down", device)
                 else:
                     devices_down.append(dev)
 
@@ -347,6 +394,7 @@ class AgentNotifierApi(dvr_rpc.DVRAgentRpcApiMixin,
         1.1 - Added get_active_networks_info, create_dhcp_port,
               update_dhcp_port, and removed get_dhcp_port methods.
         1.4 - Added network_update
+        1.5 - Added binding_activate and binding_deactivate
     """
 
     def __init__(self, topic):
@@ -363,6 +411,10 @@ class AgentNotifierApi(dvr_rpc.DVRAgentRpcApiMixin,
         self.topic_network_update = topics.get_topic_name(topic,
                                                           topics.NETWORK,
                                                           topics.UPDATE)
+        self.topic_port_binding_deactivate = topics.get_topic_name(
+            topic, n_topics.PORT_BINDING, n_topics.DEACTIVATE)
+        self.topic_port_binding_activate = topics.get_topic_name(
+            topic, n_topics.PORT_BINDING, n_topics.ACTIVATE)
 
         target = oslo_messaging.Target(topic=topic, version='1.0')
         self.client = n_rpc.get_client(target)
@@ -389,3 +441,14 @@ class AgentNotifierApi(dvr_rpc.DVRAgentRpcApiMixin,
         cctxt = self.client.prepare(topic=self.topic_network_update,
                                     fanout=True, version='1.4')
         cctxt.cast(context, 'network_update', network=network)
+
+    def binding_deactivate(self, context, port_id, host, network_id):
+        cctxt = self.client.prepare(topic=self.topic_port_binding_deactivate,
+                                    fanout=True, version='1.5')
+        cctxt.cast(context, 'binding_deactivate', port_id=port_id, host=host,
+                   network_id=network_id)
+
+    def binding_activate(self, context, port_id, host):
+        cctxt = self.client.prepare(topic=self.topic_port_binding_activate,
+                                    fanout=True, version='1.5')
+        cctxt.cast(context, 'binding_activate', port_id=port_id, host=host)
