@@ -20,25 +20,28 @@ Unit Tests for ml2 rpc
 import collections
 
 import mock
+from neutron_lib.agent import topics
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.callbacks import resources
 from neutron_lib import constants
+from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from neutron_lib.services.qos import constants as qos_consts
 from oslo_config import cfg
 from oslo_context import context as oslo_context
 from sqlalchemy.orm import exc
 
 from neutron.agent import rpc as agent_rpc
-from neutron.callbacks import resources
-from neutron.common import topics
+from neutron.common import constants as n_const
 from neutron.db import provisioning_blocks
 from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2.drivers import type_tunnel
 from neutron.plugins.ml2 import managers
 from neutron.plugins.ml2 import rpc as plugin_rpc
-from neutron.services.qos import qos_consts
 from neutron.tests import base
 
 
-cfg.CONF.import_group('ml2', 'neutron.plugins.ml2.config')
+cfg.CONF.import_group('ml2', 'neutron.conf.plugins.ml2.config')
 
 
 class RpcCallbacksTestCase(base.BaseTestCase):
@@ -50,7 +53,7 @@ class RpcCallbacksTestCase(base.BaseTestCase):
         self.callbacks = plugin_rpc.RpcCallbacks(self.notifier,
                                                  self.type_manager)
         self.plugin = mock.MagicMock()
-        directory.add_plugin(constants.CORE, self.plugin)
+        directory.add_plugin(plugin_constants.CORE, self.plugin)
 
     def _test_update_device_up(self, host=None):
         kwargs = {
@@ -60,11 +63,10 @@ class RpcCallbacksTestCase(base.BaseTestCase):
         }
         with mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin'
                         '._device_to_port_id'),\
-            mock.patch.object(self.callbacks, 'notify_ha_port_status'):
+            mock.patch.object(self.callbacks, 'notify_l2pop_port_wiring'):
             with mock.patch('neutron.db.provisioning_blocks.'
                             'provisioning_complete') as pc:
-                with mock.patch.object(ml2_db, 'get_port'):
-                    self.callbacks.update_device_up(mock.Mock(), **kwargs)
+                self.callbacks.update_device_up(mock.Mock(), **kwargs)
                 return pc
 
     def test_update_device_up_notify(self):
@@ -154,6 +156,14 @@ class RpcCallbacksTestCase(base.BaseTestCase):
         res = self.callbacks.get_device_details(mock.Mock(), host='fake')
         self.assertEqual('test-policy-id', res['network_qos_policy_id'])
 
+    def test_get_device_details_port_no_active_in_host(self):
+        port = collections.defaultdict(lambda: 'fake_port')
+        self.plugin.get_bound_port_context().current = port
+        port['device_owner'] = constants.DEVICE_OWNER_COMPUTE_PREFIX
+        port[portbindings.HOST_ID] = 'other-host'
+        res = self.callbacks.get_device_details(mock.Mock(), host='host')
+        self.assertIn(n_const.NO_ACTIVE_BINDING, res)
+
     def test_get_device_details_qos_policy_id_from_port(self):
         port = collections.defaultdict(
             lambda: 'fake_port',
@@ -179,10 +189,10 @@ class RpcCallbacksTestCase(base.BaseTestCase):
             f.assert_has_calls(calls)
 
     def test_get_devices_details_list(self):
-        devices = [1, 2, 3, 4, 5]
-        expected = devices
+        results = [{'device': [v]} for v in [1, 2, 3, 4, 5]]
+        expected = results
         callback = self.callbacks.get_devices_details_list
-        self._test_get_devices_list(callback, devices, expected)
+        self._test_get_devices_list(callback, results, expected)
 
     def test_get_devices_details_list_with_empty_devices(self):
         with mock.patch.object(self.callbacks, 'get_device_details') as f:
@@ -214,7 +224,7 @@ class RpcCallbacksTestCase(base.BaseTestCase):
 
     def _test_update_device_not_bound_to_host(self, func):
         self.plugin.port_bound_to_host.return_value = False
-        self.callbacks.notify_ha_port_status = mock.Mock()
+        self.callbacks.notify_l2pop_port_wiring = mock.Mock()
         self.plugin._device_to_port_id.return_value = 'fake_port_id'
         res = func(mock.Mock(), device='fake_device', host='fake_host')
         self.plugin.port_bound_to_host.assert_called_once_with(mock.ANY,
@@ -238,7 +248,7 @@ class RpcCallbacksTestCase(base.BaseTestCase):
 
     def test_update_device_down_call_update_port_status(self):
         self.plugin.update_port_status.return_value = False
-        self.callbacks.notify_ha_port_status = mock.Mock()
+        self.callbacks.notify_l2pop_port_wiring = mock.Mock()
         self.plugin._device_to_port_id.return_value = 'fake_port_id'
         self.assertEqual(
             {'device': 'fake_device', 'exists': False},
@@ -317,7 +327,7 @@ class RpcCallbacksTestCase(base.BaseTestCase):
 class RpcApiTestCase(base.BaseTestCase):
 
     def _test_rpc_api(self, rpcapi, topic, method, rpc_method, **kwargs):
-        ctxt = oslo_context.RequestContext(user='fake_user',
+        ctxt = oslo_context.RequestContext(user_id='fake_user',
                                            tenant='fake_project')
         expected_retval = 'foo' if rpc_method == 'call' else None
         expected_version = kwargs.pop('version', None)
