@@ -22,8 +22,9 @@ import sys
 import time
 
 import eventlet.wsgi
-from neutron.conf import wsgi as wsgi_config
+from neutron_lib import context
 from neutron_lib import exceptions as exception
+from neutron_lib import worker as neutron_worker
 from oslo_config import cfg
 import oslo_i18n
 from oslo_log import log as logging
@@ -38,12 +39,11 @@ import six
 import webob.dec
 import webob.exc
 
-from neutron._i18n import _, _LE, _LI
+from neutron._i18n import _
 from neutron.common import config
 from neutron.common import exceptions as n_exc
-from neutron import context
+from neutron.conf import wsgi as wsgi_config
 from neutron.db import api
-from neutron import worker as neutron_worker
 
 CONF = cfg.CONF
 wsgi_config.register_socket_opts()
@@ -59,7 +59,7 @@ def encode_body(body):
     return encodeutils.to_utf8(body)
 
 
-class WorkerService(neutron_worker.NeutronWorker):
+class WorkerService(neutron_worker.BaseWorker):
     """Wraps a worker to be handled by ProcessLauncher"""
     def __init__(self, service, application, disable_ssl=False,
                  worker_process_count=0):
@@ -129,7 +129,7 @@ class Server(object):
             family = info[0]
             bind_addr = info[-1]
         except Exception:
-            LOG.exception(_LE("Unable to listen on %(host)s:%(port)s"),
+            LOG.exception("Unable to listen on %(host)s:%(port)s",
                           {'host': host, 'port': port})
             sys.exit(1)
 
@@ -192,8 +192,8 @@ class Server(object):
             # The API service runs in a number of child processes.
             # Minimize the cost of checking for child exit by extending the
             # wait interval past the default of 0.01s.
-            self._server = common_service.ProcessLauncher(cfg.CONF,
-                                                          wait_interval=1.0)
+            self._server = common_service.ProcessLauncher(
+                cfg.CONF, wait_interval=1.0, restart_method='mutate')
             self._server.launch_service(service,
                                         workers=service.worker_process_count)
 
@@ -242,13 +242,13 @@ class Request(wsgi.Request):
             if _format in ['json']:
                 return 'application/{0}'.format(_format)
 
-        #Then look up content header
+        # Then look up content header
         type_from_header = self.get_content_type()
         if type_from_header:
             return type_from_header
         ctypes = ['application/json']
 
-        #Finally search in Accept-* headers
+        # Finally search in Accept-* headers
         bm = self.accept.best_match(ctypes)
         return bm or 'application/json'
 
@@ -411,7 +411,7 @@ class RequestDeserializer(object):
         """Extract necessary pieces of the request.
 
         :param request: Request object
-        :returns tuple of expected controller action name, dictionary of
+        :returns: tuple of expected controller action name, dictionary of
                  keyword arguments to pass to the controller, the expected
                  content type of the response
 
@@ -583,27 +583,27 @@ class Resource(Application):
     def __call__(self, request):
         """WSGI method that controls (de)serialization and method dispatch."""
 
-        LOG.info(_LI("%(method)s %(url)s"),
+        LOG.info("%(method)s %(url)s",
                  {"method": request.method, "url": request.url})
 
         try:
             action, args, accept = self.deserializer.deserialize(request)
         except exception.InvalidContentType:
             msg = _("Unsupported Content-Type")
-            LOG.exception(_LE("InvalidContentType: %s"), msg)
+            LOG.exception("InvalidContentType: %s", msg)
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
         except n_exc.MalformedRequestBody:
             msg = _("Malformed request body")
-            LOG.exception(_LE("MalformedRequestBody: %s"), msg)
+            LOG.exception("MalformedRequestBody: %s", msg)
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
 
         try:
             action_result = self.dispatch(request, action, args)
         except webob.exc.HTTPException as ex:
-            LOG.info(_LI("HTTP exception thrown: %s"), ex)
+            LOG.info("HTTP exception thrown: %s", ex)
             action_result = Fault(ex, self._fault_body_function)
         except Exception:
-            LOG.exception(_LE("Internal error"))
+            LOG.exception("Internal error")
             # Do not include the traceback to avoid returning it to clients.
             action_result = Fault(webob.exc.HTTPServerError(),
                                   self._fault_body_function)
@@ -616,10 +616,10 @@ class Resource(Application):
             response = action_result
 
         try:
-            LOG.info(_LI("%(url)s returned with HTTP %(status)d"),
+            LOG.info("%(url)s returned with HTTP %(status)d",
                      dict(url=request.url, status=response.status_int))
         except AttributeError as e:
-            LOG.info(_LI("%(url)s returned a fault: %(exception)s"),
+            LOG.info("%(url)s returned a fault: %(exception)s",
                      dict(url=request.url, exception=e))
 
         return response
@@ -629,11 +629,11 @@ class Resource(Application):
 
         controller_method = getattr(self.controller, action)
         try:
-            #NOTE(salvatore-orlando): the controller method must have
+            # NOTE(salvatore-orlando): the controller method must have
             # an argument whose name is 'request'
             return controller_method(request=request, **action_args)
         except TypeError:
-            LOG.exception(_LE('Invalid request'))
+            LOG.exception('Invalid request')
             return Fault(webob.exc.HTTPBadRequest())
 
 
